

rm(list = ls())
library(tidyverse)
library(interp)
library(readxl)
library(writexl)
library(pammtools)
library(patchwork)
dir <-  dirname(rstudioapi::getSourceEditorContext()$path)
setwd(dir)
source("volc.freq.R")

HL09 <- read_excel(
  "eruption.database.intcal20.xlsx",
  sheet = "HL09",
  col_types = c(rep("numeric", 7), "text", "numeric", "text"),
  na = "NaN"
)

DMH20 <-
  read_excel(
    "eruption.database.intcal20.xlsx",
    sheet = "GVP.B06.LaMEVE.AVO.W13.AK",
    col_types = c(
      "numeric",
      "text",
      rep("numeric", 5),
      "text",
      "numeric",
      "text",
      "numeric",
      "text"
    ),
    na = "NA"
  )


hl09.res <- volcfreq(HL09, dage2 = 2, norm = "L")

dmh.res <- volcfreq(DMH20, dage2 = 2, norm = "L")


freq.summary <- hl09.res[[1]] %>% mutate(ref = "HL09") %>%
  full_join(dmh.res[[1]] %>% mutate(ref = "This study"))

eruption.ratio.summary <- hl09.res[[2]] %>% mutate(ref = "HL09") %>%
  full_join(dmh.res[[2]] %>% mutate(ref = "This study"))


write_xlsx(
  list(frequency = freq.summary, eruption.ratio = eruption.ratio.summary),
  "Volcanic eruption summary.xlsx"
)
