


volcfreq <- function(database, dage2=2, norm="L") {


  volc.freq <- database %>%
    filter(!is.na(group)) %>%
    mutate(index = 1:length(age)) %>%
    mutate(index = paste0("JD",index))


  if (norm == "H") {
    binbreaks  = seq(-0.07, 40, dage2)
  } else if (norm == "L") {
    binbreaks = seq(-0.07, 40, dage2)#seq(0, 40, dage2)
  }

  if ((norm == "H") & (dage2 == 2)) {
    norm.bin <- c("-0.07~1.93")
  }  else if ((norm == "L") & (dage2 == 2)) {
    norm.bin = c("21.93~23.93")#
  }



  binfun <- function(age, age.sd, method) {
    age.rand <- rnorm(1000, mean=age, sd=age.sd)
    bin <- cut(age.rand, breaks = binbreaks, dig.lab = 6)
    return(bin)
  }

  input.age <- volc.freq$age
  names(input.age) <-volc.freq$index

  rand.age.bin <- mapply(binfun, input.age, volc.freq$age.sd,
                         SIMPLIFY = TRUE,USE.NAMES=TRUE) %>%
    data.frame() %>%
    rownames_to_column("rand") %>%
    gather(key="index",value="age_bin",-rand) %>%
    filter(!is.na(age_bin))

  volc.freq.rand <-  rand.age.bin %>%
    left_join(volc.freq) %>%
    ungroup() %>%
    group_by(rand,group,age_bin)%>%
    summarise(freq = n()) %>%
    filter(!is.na(age_bin)) %>%
    separate(age_bin, c("age_l", "age_m"), ",") %>%
    mutate(age_l = parse_number(age_l)) %>%
    mutate(age_m = parse_number(age_m)) %>%
    mutate(age_bin = paste0(age_l, "~", age_m)) %>%
    mutate(age_bin = factor(age_bin))



  volc.freq.rand.NP <-rand.age.bin %>%
    left_join(volc.freq) %>%
      mutate(
        group = ifelse(
          lat >= 45 &
            lat <= 70 & lon >= 180 & lon <= 240 & group == "glaciated",
          "NP.glaciated",
          ifelse(group == "glaciated", "glaciated.r", "unglaciated")
        )
      ) %>%
    group_by(rand,group,age_bin)%>%
    summarise(freq = n()) %>%
    filter(!is.na(age_bin)) %>%
    separate(age_bin, c("age_l", "age_m"), ",") %>%
    mutate(age_l = parse_number(age_l)) %>%
    mutate(age_m = parse_number(age_m)) %>%
    mutate(age_bin = paste0(age_l, "~", age_m)) %>%
    mutate(age_bin = factor(age_bin))



  old.levels <- levels(as.factor(volc.freq.rand$age_bin))
  levels <- old.levels[order(parse_number(old.levels))]

  volc.freq.rand.norm = volc.freq.rand %>%
    spread(key = group, value = freq) %>%
    mutate(g.ung = glaciated / unglaciated) %>%
    ungroup() %>%
    group_by(rand) %>%
    mutate(denom = mean(g.ung[age_bin %in% norm.bin],na.rm=TRUE)) %>%
    mutate(g.ung_n = (g.ung) / denom) %>%
    mutate(age_bin = factor(age_bin, levels))


  volc.freq.rand.NP.norm = volc.freq.rand.NP %>%
    spread(key = group, value = freq) %>%
    mutate(g.ung.NP = NP.glaciated / unglaciated) %>%
    mutate(g.ung.r = glaciated.r / unglaciated) %>%
    ungroup() %>%
    group_by(rand) %>%
    mutate(denom.NP = mean(g.ung.NP[age_bin %in% norm.bin],na.rm=TRUE)) %>%
    mutate(denom.r = mean(g.ung.r[age_bin %in% norm.bin],na.rm=TRUE)) %>%
    mutate(g.ung.NP = g.ung.NP / denom.NP) %>%
    mutate(g.ung.r = g.ung.r / denom.r) %>%
    mutate(age_bin = factor(age_bin, levels))


  volc.freq.rand.summary  <-  volc.freq.rand.norm %>%
    ungroup() %>%
    select(age_l, age_m, age_bin, g.ung_n) %>%
    group_by(age_l, age_m, age_bin) %>%
    summarise(
      g.ung.m = median(g.ung_n, na.rm = TRUE),
      g.ung.l = quantile(g.ung_n, 0.25, na.rm = TRUE),
      g.ung.u = quantile(g.ung_n, 0.75, na.rm = TRUE)
    )

  volc.freq.rand.NP.summary  <-  volc.freq.rand.NP.norm %>%
    ungroup() %>%
    select(age_l, age_m, age_bin, g.ung.NP, g.ung.r) %>%
    gather(key = "group", value = "g.ung_n", -age_l, -age_m, -age_bin) %>%
    group_by(age_l, age_m, age_bin, group) %>%
    summarise(
      g.ung.m = median(g.ung_n, na.rm = TRUE),
      g.ung.l = quantile(g.ung_n, 0.25, na.rm = TRUE),
      g.ung.u = quantile(g.ung_n, 0.75, na.rm = TRUE)
    )

  eruption.ratio.norm <- volc.freq.rand.summary %>%
    mutate(group="g.ung.global") %>%
    full_join(volc.freq.rand.NP.summary)

  volc.freq.summary <- volc.freq.rand %>%
    group_by(group, age_l, age_m, age_bin) %>%
    summarize(
      freq.m = median(freq, na.rm = TRUE),
      freq.l = quantile(freq, 0.25, na.rm = TRUE),
      freq.u = quantile(freq, 0.75, na.rm = TRUE)
    ) %>%
    full_join(
      volc.freq.rand.NP %>% filter(group != "unglaciated") %>%
        group_by(group, age_l, age_m, age_bin) %>%
        summarize(
          freq.m = median(freq, na.rm = TRUE),
          freq.l = quantile(freq, 0.25, na.rm = TRUE),
          freq.u = quantile(freq, 0.75, na.rm = TRUE)
        )
    )


  return(list(volc.freq.summary, eruption.ratio.norm))
}