
rm(list=ls())
library(ncdf4)
library(tidyverse)
library(readxl)
library(writexl)
library(pals)
library(patchwork)
dir <-
  paste0(dirname(rstudioapi::getSourceEditorContext()$path), "/")
setwd(dir)


tend = 120e3

epica <-
  read_excel("temperature timeseries.xlsx", sheet = "epica") %>%
  filter(!is.na(delT)) %>%
  select(age_yrbp, delT) %>%
  filter(age_yrbp <= tend) %>%
  mutate(model = "epica")

grip <-
  read_excel("temperature timeseries.xlsx", sheet = "grip") %>%
  filter(!is.na(delT)) %>%
  select(age_yrbp, delT) %>%
  filter(age_yrbp <= tend) %>%
  mutate(model = "grip")

ngrip <-
  read_excel("temperature timeseries.xlsx", sheet = "ngrip") %>%
  filter(!is.na(delT)) %>%
  select(age_yrbp, delT) %>%
  filter(age_yrbp <= tend) %>%
  mutate(model = "ngrip")


vostok <-
  read_excel("temperature timeseries.xlsx", sheet = "vostok") %>%
  filter(!is.na(delT)) %>%
  select(age_yrbp, delT) %>%
  filter(age_yrbp <= tend) %>%
  mutate(model = "vostok")

odp1020 <-
  read_excel("temperature timeseries.xlsx", sheet = "1020") %>%
  filter(!is.na(delT)) %>%
  select(age_yrbp, delT) %>%
  filter(age_yrbp <= tend) %>%
  mutate(model = "odp1020")

odp1012 <-
  read_excel("temperature timeseries.xlsx", sheet = "1012") %>%
  filter(!is.na(delT)) %>%
  select(age_yrbp, delT) %>%
  filter(age_yrbp <= tend) %>%
  mutate(model = "odp1012")


temperature.data = epica %>%
  full_join(grip) %>%
  full_join(ngrip) %>%
  full_join(vostok) %>%
  full_join(odp1020) %>%
  full_join(odp1012)


readdata <- function(file) {
  data <- nc_open(file)
  age <- rev(ncvar_get(data, "age"))
  ice <- rev(ncvar_get(data, "slvol"))
  dvoldt <-
    rev(ncvar_get(data, "divoldt")) * 365 * 24 * 60 * 60 / 1e3/361900000 # m/kyr
  return(data.frame(
    age = age,
    ice.volume.msle = ice,
    dvoldt.msle.kyr = dvoldt
  ))
  nc_close(data)

}

epica.5km <-
  readdata(paste0(dir, "/ciscyc.5km.epica.ts.10a.nc")) %>%
  mutate(model = "epica", resolution = "5km")
epica.10km <-
  readdata(paste0(dir, "/ciscyc.10km.epica.ts.10a.nc")) %>%
  mutate(model = "epica", resolution = "10km")

grip.10km <-
  readdata(paste0(dir, "/ciscyc.10km.grip.ts.10a.nc")) %>%
  mutate(model = "grip", resolution = "10km")
grip.5km <-
  readdata(paste0(dir, "/ciscyc.5km.grip.ts.10a.nc")) %>%
  mutate(model = "grip", resolution = "5km")

ngrip.10km <-
  readdata(paste0(dir, "/ciscyc.10km.ngrip.ts.10a.nc")) %>%
  mutate(model = "ngrip", resolution = "10km")

vostok.10km <-
  readdata(paste0(dir, "/ciscyc.10km.vostok.ts.10a.nc")) %>%
  mutate(model = "vostok", resolution = "10km")

odp1020.10km <-
  readdata(paste0(dir, "/ciscyc.10km.odp1020.ts.10a.nc")) %>%
  mutate(model = "odp1020", resolution = "10km")
odp1012.10km <-
  readdata(paste0(dir, "/ciscyc.10km.odp1012.ts.10a.nc")) %>%
  mutate(model = "odp1012", resolution = "10km")


ice.data <- epica.5km %>%
  full_join(epica.10km) %>%
  full_join(grip.5km) %>%
  full_join(grip.10km) %>%
  full_join(ngrip.10km) %>%
  full_join(vostok.10km) %>%
  full_join(odp1012.10km) %>%
  full_join(odp1020.10km) %>%
  mutate(age = age * 1e3)



age <- seq(0,120e3,1)


delT.epica <- approx(x = epica$age_yrbp,
                     y = epica$delT,
                     xout = age)$y

delT.grip <- approx(x = grip$age_yrbp,
                    y = grip$delT,
                    xout = age)$y

delT.ngrip <- approx(x = ngrip$age_yrbp,
                     y = ngrip$delT,
                     xout = age)$y

delT.vostok <- approx(x = vostok$age_yrbp,
                      y = vostok$delT,
                      xout = age)$y
delT.odp1012 <- approx(x = odp1012$age_yrbp,
                       y = odp1012$delT,
                       xout = age)$y

delT.odp1020 <- approx(x = odp1020$age_yrbp,
                       y = odp1020$delT,
                       xout = age)$y


step <- 10
bins <- seq(-step / 2-100, 125e3, step)


temperature.data.binned <-
  data.frame(age,
             delT.epica,
             delT.grip,
             delT.ngrip,
             delT.vostok,
             delT.odp1012,
             delT.odp1020) %>%
  gather(-age, key = "model", val = "delT") %>%
  separate(model, c(NA, "model")) %>%
  mutate(bins = cut(age, breaks = bins)) %>%
  group_by(bins, model) %>%
  summarize(
    delT = mean(delT, na.rm = TRUE)
  ) %>%
  mutate(ageL = as.numeric(str_extract(
    bins, regex("(?<=\\()[-]*[\\w\\+\\.]+(?=,)")
  ))) %>%
  mutate(ageU = as.numeric(str_extract(
    bins, regex("(?<=,)[-]*[\\w\\+\\.]+(?=\\])")
  ))) %>%
  mutate(age = (ageL + ageU) / 2) %>%
  ungroup() %>%
  select(-bins, -ageL, -ageU) %>%
  mutate(model = toupper(model))


ggplot(temperature.data.binned) +
  geom_line(aes(age, delT, color = model))+
  facet_grid(model~.)



ice.data.binned <- ice.data %>%
  mutate(bins = cut(age, breaks = bins)) %>%
  group_by(bins, model, resolution) %>%
  summarize(
    ice.volume.msle = mean(ice.volume.msle, na.rm = TRUE),
    dvoldt.msle.kyr = mean(dvoldt.msle.kyr, na.rm = TRUE)
  ) %>%
  mutate(ageL = as.numeric(str_extract(
    bins, regex("(?<=\\()[-]*[\\w\\+\\.]+(?=,)")
  ))) %>%
  mutate(ageU = as.numeric(str_extract(
    bins, regex("(?<=,)[-]*[\\w\\+\\.]+(?=\\])")
  ))) %>%
  mutate(age = (ageL + ageU) / 2) %>%
  ungroup() %>%
  select(-bins, -ageL, -ageU) %>%
  mutate(model = toupper(model)) %>%
  mutate(label = paste0(model, ".", resolution)) %>%
  left_join(temperature.data.binned)

ggplot(ice.data.binned,aes(age,ice.volume.msle,color=model))+
  geom_line() +
  facet_grid(label~.)
ggplot(ice.data.binned,aes(age,dvoldt.msle.kyr,color=model))+
  geom_line()+
  facet_grid(label~.)
ggplot(ice.data.binned,aes(age,delT,color=model))+
  geom_line()+
  facet_grid(label~.)

ggplot(ice.data.binned %>%
         filter(age<=24e3),aes(delT,ice.volume.msle,color=model))+
  geom_point()+
  geom_smooth(method="lm")




ice.data.binned.check <- ice.data.binned %>%
  group_by(label, model, resolution) %>%
  arrange(age) %>%
  filter(!is.na(delT)) %>%
  filter(age >= 6e3, age <= 30e3) %>%
  summarize(step = paste(unique(diff(age)), collapse = "/") == step,
            nstep = length(unique(diff(age))))






lagcor <- function(label.i, lims) {
  data <- ice.data.binned %>%
    filter(label == label.i) %>%
    filter(!is.na(delT)) %>%
    filter(age >= lims[1], age <= lims[2]) %>%
    ungroup() %>%
    arrange(age)

  data.check <- data %>%
    arrange(age) %>%
    summarize(step = paste(unique(diff(age)), collapse = "/") == step,
              nstep = length(unique(diff(age))))

  if ((!data.check$step) | (data.check$nstep != 1)) {
    stop("Incorrect time steps")
  }

  fit = ccf(
    data$delT,
    data$ice.volume.msle,
      lag = 5e4 / step,
      plot = TRUE
    )

  output <- data.frame(label=label.i,cor = fit$acf,lag.yr=fit$lag*step) %>%
    mutate(min.cor = min(cor)) %>%
    mutate(lag.best.yr = lag.yr[cor == min(cor)])

  return(output)

}

age1=0e3
age2=26e3
age3=6e3
age4=32e3

EPICA.10km.lag <- lagcor("EPICA.10km",c(age1,age2))
EPICA.5km.lag <- lagcor("EPICA.5km",c(age1,age2))
GRIP.10km.lag <- lagcor("GRIP.10km",c(age1,age2))
GRIP.5km.lag <- lagcor("GRIP.5km",c(age1,age2))
NGRIP.10km.lag <- lagcor("NGRIP.10km",c(age1,age2))
VOSTOK.10km.lag <- lagcor("VOSTOK.10km",c(age1,age2))
ODP1012.10km.lag <- lagcor("ODP1012.10km",c(age3,age4))
ODP1020.10km.lag <- lagcor("ODP1020.10km",c(age1,age2))



lag.cor <- EPICA.10km.lag %>%
  full_join(EPICA.5km.lag) %>%
  full_join(GRIP.10km.lag) %>%
  full_join(GRIP.5km.lag) %>%
  full_join(NGRIP.10km.lag) %>%
  full_join(VOSTOK.10km.lag) %>%
  full_join(ODP1012.10km.lag) %>%
  full_join(ODP1020.10km.lag)

lags <- lag.cor %>%
  select(label, lag.best.yr ) %>%
  distinct()

mean(lags$lag.best.yr)
sd(lags$lag.best.yr)

mean(lags$lag.best.yr[c(1:6,8)])
sd(lags$lag.best.yr[c(1:6,8)])


step2 <- 10
bins2 <- seq(-step / 2-100, 125e3, step2)

lagged.timeseries  <- ice.data.binned %>%
  left_join(lags) %>%
  group_by(label,model,resolution) %>%
  mutate(delT.lag = approx(age, delT, xout = age + lag.best.yr)$y) %>%
  mutate(bins = cut(age, breaks = bins2)) %>%
  group_by(bins, model,resolution,label) %>%
  summarize(
    delT = mean(delT, na.rm = TRUE),
    delT.lag = mean(delT.lag, na.rm = TRUE),
    ice.volume.msle = mean(ice.volume.msle,na.rm = TRUE),
    dvoldt.msle.kyr = mean(dvoldt.msle.kyr,na.rm = TRUE),
    lag.best.yr = unique(lag.best.yr,na.rm=TRUE)
  ) %>%
  mutate(ageL = as.numeric(str_extract(
    bins, regex("(?<=\\()[-]*[\\w\\+\\.]+(?=,)")
  ))) %>%
  mutate(ageU = as.numeric(str_extract(
    bins, regex("(?<=,)[-]*[\\w\\+\\.]+(?=\\])")
  ))) %>%
  mutate(age = (ageL + ageU) / 2) %>%
  ungroup() %>%
  select(-bins, -ageL, -ageU)


ggplot(lagged.timeseries %>%
         gather(key = "type", value = "delT", delT, delT.lag)) +
  geom_line(aes(age, delT, color = type)) +
  facet_grid(label~.)+
  scale_x_continuous(limits=c(0,30e3))



lagged.regression <- lagged.timeseries %>%
  filter(age>=0e3,age<=24e3) %>%
  group_by(label,model,resolution) %>%
  nest() %>%
  mutate(fit = purrr::map(.x = data,
                   .f =  ~ lm(ice.volume.msle ~ delT.lag, .x))) %>%
  mutate(stat = purrr::map(.x = fit, summary)) %>%
  mutate(coeff = purrr::map(.x = fit, pluck, 1)) %>%
  mutate(slope = purrr::map(.x = coeff, pluck, 2)) %>%
  # mutate(intercept = purrr::map(.x = coeff, pluck, 1)) %>%
  mutate(r.sq = purrr::map(.x = stat, pluck, 9)) %>%
  select(label,model, resolution, slope,r.sq) %>%
  mutate(slope = unlist(slope)) %>%
  mutate(slope = round(slope, digits = 2)) %>%
  # mutate(intercept = unlist(intercept)) %>%
  # mutate(intercept = round(intercept, digits = 2)) %>%
  mutate(r.sq = unlist(r.sq)) %>%
  mutate(r.sq = round(r.sq, digits = 2))


mean(lagged.regression$slope)
sd(lagged.regression$slope)


ggplot(
  lagged.timeseries %>% filter(age >= 0e3, age <= 24e3),
  aes(delT.lag, ice.volume.msle, color = model)
) +
  geom_point() +
  geom_smooth(method = "lm",
              se = TRUE,
              formula = y ~ x)




write_xlsx(
  list(
    temperature.forcing = temperature.data,
    modeled.ice.volume = ice.data,
    binned.data = ice.data.binned,
    lag.cor = lag.cor,
    lags = lags,
    lagged.timeseries = lagged.timeseries,
    lagged.regression = lagged.regression
  ),
  "PISM sensitivity.xlsx"
)

